class PurseType(object):
    TYPE_WEBMONEY = 'WebMoney'          # WebMoney
    TYPE_YANDEX_MONEY = 'Yandex'        # Яндекс.Деньги
    TYPE_QIWI = 'QIWI'                  # QIWI
    TYPE_QIWI_SE = 'QIWI_SE'            # QIWI карты самозанятых
    TYPE_ZALEY_CASH = 'ZaleyCash'       # ZaleyCash
    TYPE_RU_CARD = 'RUCARD'             # Банковская карта РФ
    TYPE_WORLD_CARD = 'WORLDCARD'       # Банковская карта Mastercard Worldwide
    TYPE_SNG_CARD = 'SNGCARD'           # Банковская карта MasterCard СНГ USD
    TYPE_KZ_CARD = 'KZCARD'             # Банковская карта Казахстана
    TYPE_UKR_CARD = 'UKRCARD'           # Банковская карта Украины
    TYPE_MTS = 'MTS'                    # MTS
    TYPE_BEELINE = 'Beeline'            # Билайн
    TYPE_MEGAFON = 'MegaFon'            # Мегафон
    TYPE_TELE2 = 'Tele2'                # Tele2
    TYPE_CAPITALIST = 'Capitalist'      # Capitalist
    TYPE_USDTTRC20 = 'USDTTRC20'        # USDT (Tron)
    TYPE_USDTERC20 = 'USDTERC20'        # USDT (Etherium)
    TYPE_PAYTM = 'PAYTM'                # PayTM
    TYPE_IMPS = 'IMPS'                  # IMPS


class Currency(object):
    RUB = 'RUB'
    USD = 'USD'
    EURO = 'EUR'
    INR = 'INR'



import json
import os
from decimal import Decimal
from urllib.parse import urlencode

import requests

from .settings import MASTER_PAY_SETTINGS


class MasterPayApiException(Exception):

    def __init__(self, code, error, status_code):
        self.code = code
        self.error = error
        self.status_code = status_code
        super(MasterPayApiException, self).__init__("code: {}, error: {}".format(self.code, self.error))


class MasterPayApi(object):

    def __init__(self, token=MASTER_PAY_SETTINGS['token']):
        self.token = token
        self.session = requests.Session()

    def _make_request(self, method, url, *args, **kwargs):
        if 'timeout' not in kwargs:
            kwargs['timeout'] = MASTER_PAY_SETTINGS['timeout']

        try:
            response = getattr(self.session, method.lower())(url, *args, **kwargs)
            response.raise_for_status()
        except requests.HTTPError as e:
            try:
                error_data = json.loads(e.response.content)
                raise MasterPayApiException(error_data['code'], error_data['error'], e.response.status_code)
            except json.JSONDecodeError:
                raise e
        json_data = response.json()
        if not json_data['success']:
            raise MasterPayApiException(json_data['code'], json_data['error'], response.status_code)

        return json_data['result']

    def create_payment(
            self,
            amount,
            purse_type,
            currency,
            purse_number,
            external_id,
            partner_id=MASTER_PAY_SETTINGS['default_partner_id'],
            **extra_params
    ):
        url = os.path.join(MASTER_PAY_SETTINGS['base_url'], 'api', 'partner', str(partner_id), 'payment', 'create/')
        extra = extra_params or {}
        params = {
            'amount': Decimal(amount),
            'purse_type': purse_type,
            'currency': currency,
            'number': purse_number,
            'external_id': external_id,
            **extra
        }

        url = url + "?{}".format(urlencode({'token': self.token}))
        data = self._make_request('post', url, data=params)
        return data

    def get_payment(self, payment_id, partner_id=MASTER_PAY_SETTINGS['default_partner_id']):
        url = os.path.join(MASTER_PAY_SETTINGS['base_url'], 'api', 'partner', str(partner_id), 'payment', str(payment_id), 'detail')
        data = self._make_request('get', "{}?{}".format(url, urlencode({'token': self.token})))
        return data



from django.db import transaction

from django_master_pay.api import MasterPayApi
from django_master_pay.settings import MASTER_PAY_SETTINGS


def create_payment(
        amount,
        currency,
        purse_type,
        purse_number,
        external_id,
        partner_id=MASTER_PAY_SETTINGS['default_partner_id'],
        token=MASTER_PAY_SETTINGS['token'],
        **extra_params
):
    from django_master_pay.models import Payment
    api = MasterPayApi(token=token)

    with transaction.atomic():
        payment_data = api.create_payment(
            amount,
            purse_type,
            currency,
            purse_number,
            external_id,
            partner_id=partner_id,
            **extra_params
        )
        payment = Payment(
            external_id=external_id,
            master_pay_id=payment_data['id'],
            record_data=payment_data,
            partner_id=partner_id
        )
        payment.save()
        return payment



import logging

try:
    from django.conf import settings as django_settings
except ImportError:
    logging.warning("Django not installed")
    django_settings = None


def default_callback(**kwargs):
    raise NotImplemented("MasterPay callback not implemented")


MASTER_PAY_SETTINGS = {
    'token': '',
    'callback': default_callback,
    'default_partner_id': None,
    'base_url': "https://master-pay.ru/",
    'timeout': 30,
    'logger_name': 'django_master_pay'
}

if hasattr(django_settings, 'MASTER_PAY_SETTINGS'):
    MASTER_PAY_SETTINGS.update(django_settings.MASTER_PAY_SETTINGS)



from .utils import create_payment

__title__ = 'Django Master Pay'
__version__ = "1.0.7"
__author__ = 'KokocGroup'
__license__ = 'BSD 2-Clause'

VERSION = __version__

default_app_config = 'django_master_pay.apps.DjangoMasterPayConfig'



# Generated by Django 2.2.2 on 2021-07-19 15:17

import django.contrib.postgres.fields.jsonb
from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Payment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(default=django.utils.timezone.now, verbose_name='Создан')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Обновлен')),
                ('external_id', models.CharField(max_length=250, verbose_name='Внешний ключ')),
                ('master_pay_id', models.CharField(max_length=250, verbose_name='Индефикатор MasterPay')),
                ('status', models.CharField(choices=[('NEW', 'Новый'), ('IN_WORK', 'В работе'), ('DONE', 'Проведен'), ('ERROR', 'Ошибка')], default='NEW', max_length=100, verbose_name='Статус')),
                ('record_data', django.contrib.postgres.fields.jsonb.JSONField(verbose_name='RAW data')),
            ],
            options={
                'verbose_name': 'Платеж',
                'verbose_name_plural': 'Платежи',
            },
        ),
    ]
